# General information:
# 1) We will assume for the sake of simplicity that the data fulfill all requirements 
# for the use of t tests: Observations are randomly and independently drawn from 
# normally distributed populations of equal variance.
# 2) The 95% CIs refer only to the individual mean (arelational CI) and cannot be used
# directly to draw conclusions about differences between means (i.e., do not use the CIs 
# to compare different contrasts with each other).


# Example of contrast analyses of one-way within-subject data
#
# Data similar to that in Howell (2013, table 13.2) who modelled it after a study by Eysenck (1974).
# Number of recalled words in relation to five levels of encoding, defined according to  
# instructions to participants, and ordered from shallow to deep 
# (C = count letters, R = rhyme, A = think of appropriate adjective, 
#  I = imagine visually, M = memorize for later recall).
#
# Here we will treat the data as a one-way within-subject design with 20
# participants for each of the five encoding conditions
#
# Stefan Wiens & Mats E. Nilsson, 2016-06-23
# Contact: mats.nilsson@psychology.su.se


# Clear everything -------------------------------------------------------------
rm(list = ls()) # clear memory
graphics.off() # clear all plots
cat("\014")     # clear console (same as Ctrl-L in console)
#-------------------------------------------------------------------------------


# Set working directory (mydir) ------------------------------------------------
mydir <- 'c:/Users/MATNI/Dropbox/Statspaper with Mats/Supplement/R'
setwd(mydir)
# ------------------------------------------------------------------------------


# Load functions ---------------------------------------------------------------
# The file 'functions_contrasts.r' (should be in mydir) contain code for these 
# functions: get_example_data(), get_contrast(), plot_contrast()
source('functions_contrasts.r')  
# ------------------------------------------------------------------------------


# Get example data, taken from Howell, Table 13.2 ------------------------------
d <- get_example_data(design = 'within', way = 'one')
# ------------------------------------------------------------------------------


# Screening of data -------------------------------------------------------------
summary(d)
boxplot(d$recall ~ d$encode)
aggregate(d$recall, list(d$encode), summary)
# ------------------------------------------------------------------------------


# Define contrast-weight matrix-------------------------------------------------
# Create matrix with five contrasts 
# If you run "contrast_result" below with "standardset = TRUE",
# these contrasts will be converted to standard sets (i.e. absolute weights sum to 2)
# If you run "contrast_result" below with "standardset = FALSE",
# these contrasts will be computed with the given weights.
coef <-  matrix(c(-1,  1,  0,  0,  0,  # 1. R - C       
                  -1,  0,  1,  0,  0,  # 2. A - C 
                   0,  0, -2,  1,  1,  # 3. I+M - A 
                  -3, -3,  2,  2,  2,  # 4. A+I+M - C+R
                  -2, -1,  0,  1,  2), # 5. Linear trend 

                  nrow = 5, ncol = 5, byrow = TRUE)

# Check if contrast sum to zero (within 1e-12 margin of error)
if (abs(sum(apply(coef,1,sum))) > 1e-12) stop ('Not all contrasts sum to zero!')

# Add row and column names to the contrast-weight matrix 
cnames <- c('R>C', 'A>C','I+M>A', 'A+I+M>C+R', 'Trend')
rownames(coef) <- cnames
colnames(coef) <- levels(d$encode)
#-------------------------------------------------------------------------------


# Calculate and plot contrast scores with CIs, using get_contrast() ------------
contrast_result <- apply(coef, 1, get_wcontrast, data = d$recall, 
                         group = d$encode, id = d$id, 
                         standardset = FALSE)
# If you run "contrast_result" with "standardset = TRUE",
# these contrasts will be converted to standard sets (i.e. absolute weights sum to 2)
# If you run "contrast_result" with "standardset = FALSE",
# these contrasts will be computed with the given weights.

# Transpose matrix: t(), if you prefer to have each contrast in a single row
contrast_result <- t(contrast_result)  
print(contrast_result)
plot_contrast(contrast_result)
#-------------------------------------------------------------------------------

# Example results:
#
# contrast_result <- apply(coef, 1, get_wcontrast, data = d$recall, 
#                          group = d$encode, id = d$id, 
#                          standardset = FALSE)
#
# Note that the contrast weights are not turned into standard sets (standardset = FALSE).
# For example, if you entered [+2 -2 0 0 0], this captures the difference between the first
# and second mean, but this difference is multiplied by 2.
#           Score      CI_LL     CI_UL      Tobs df            p       SEM  C  R  A I M
# R>C        0.40 -0.2333912  1.033391  1.321789 19 2.019331e-01 0.3026201 -1  1  0 0 0
# A>C        6.05  4.8888655  7.211135 10.905537 19 1.279867e-09 0.5547641 -1  0  1 0 0
# I+M>A      5.40  4.1626816  6.637318  9.134537 19 2.215683e-08 0.5911630  0  0 -2 1 1
# A+I+M>C+R 45.90 38.3888657 53.411134 12.790319 19 8.758208e-11 3.5886517 -3 -3  2 2 2
# Trend     25.95 21.2221278 30.677872 11.488038 19 5.391211e-10 2.2588714 -2 -1  0 1 2

