# General information:
# 1) We will assume for the sake of simplicity that the data fulfill all requirements 
# for the use of t tests: Observations are randomly and independently drawn from 
# normally distributed populations of equal variance.
# 2) The 95% CIs refer only to the individual mean (arelational CI) and cannot be 
#  used directly to draw conclusions about differences between means (i.e., do 
# not use the CIs to compare different contrasts with each other).


# Example of contrast analyses of two-way between-subject data
#
# Simulating data consistent with Table 1A in our paper, that is, a 2x2 design, 
# with Alcohol (yes/No) and Antabus (yes/no) as factors, and nausea rating
# on a scale from 0 to 20 as outcome variable. 

# Stefan Wiens & Mats E. Nilsson, 2016-06-23
# Contact: mats.nilsson@psychology.su.se


# Clear everything -------------------------------------------------------------
rm(list = ls()) # clear memory
graphics.off() # clear all plots
cat("\014")     # clear console (same as Ctrl-L in console)
#-------------------------------------------------------------------------------


# Set working directory (mydir) ------------------------------------------------
mydir <- 'c:/Users/MATNI/Dropbox/Statspaper with Mats/Supplement/R'
setwd(mydir)
# ------------------------------------------------------------------------------


# Load functions ---------------------------------------------------------------
# The file 'functions_contrasts.r' (should be in mydir) contain code for these 
# functions: get_contrast(), plot_contrast()
source('functions_contrasts.r')  
# ------------------------------------------------------------------------------


# Simulate data consistent with Table 1A of Wiens & Nilsson (2016) -------------
cell_means <- c(16.38, 4.20, 4.22, 4.10)  # Means from Table 1A
set.seed(1)  # Skip if you want new random samples each time
temp <- matrix(rnorm(4 * 30, sd = 2), nrow = 4, ncol = 30, byrow = TRUE)
locate <- cell_means - apply(temp, 1, mean)  # Correction factor to set means
temp <- t(temp + locate)  # Add correction factors and then transpose
nausea <- as.vector(temp)  # Turn data into vector format
alcohol <- factor(rep(c('Alco', 'noAlco'), c(60, 60)))
antabuse <- factor(rep(rep(c('Anta', 'noAnta'), 2), c(30, 30, 30, 30)))
cells <- interaction(antabuse, alcohol, sep = "+")
# ------------------------------------------------------------------------------


# Screening of data ------------------------------------------------------------
boxplot(nausea ~ cells, col = rep(c('white', 'grey'), 2), cex.axis = 0.8)
aggregate(nausea, list(alcohol, antabuse), summary)
# ------------------------------------------------------------------------------

# ------------------------------------------------------------------------------
# Means are
#                 Antabuse       
#                yes     no
# Alcohol yes   16.38,  4.20,  
#         no     4.22,  4.10
#
# Define contrast-weight matrix-------------------------------------------------
#             Antabuse       
#              yes no
# Alcohol yes   a,  b,  
#         no    c,  d
#
## Create matrix with six contrasts 
# If you run "contrast_result" below with "standardset = TRUE",
# these contrasts will be converted to standard sets (i.e. absolute weights sum to 2)
# If you run "contrast_result" below with "standardset = FALSE",
# these contrasts will be computed with the given weights.
#
# NOTE! Watch out for R sorting levels differently than you intended; first
# run levels(cells) to see R's order, then give weights in this order
#
coef <-  matrix(c( 0.5, -0.5,  # 1. Table 1.B Weights for column effects 
                   0.5, -0.5,  #    (Antabuse)
                   
                   0.5,  0.5,  # 2. Table 1.C Weights for row effects (Alcohol)
                  -0.5, -0.5, 
                   
                   0.5, -0.5,  # 3. Table 1.D Weights for interaction 
                  -0.5,  0.5,  #    (Alcohol x Antabuse)
                   
                   1.0, -1/3,  # 4. Table 1.E Weights for synergistic effects
                  -1/3, -1/3,
                  
                   0.0,  1.0,  # 5. Simple effect of Alcohol  (without Antabuse)
                   0.0, -1.0,
                  
                   0.0,  0.0,  # 6. Simple effect of Antabuse (without Alcohol)
                   1.0, -1.0), 
                nrow = 6, ncol = 4, byrow = TRUE)            
                
# Check if contrasts sum to zero (within 1e-12 margin of error)
if (abs(sum(apply(coef,1,sum))) > 1e-12) stop ('Not all contrasts sum to zero!')

# Add row and column names to the contrast-weight matrix 
cnames <- c('Alco', 'Anta','Alco x Anta', 'Synergistic', 'Alco (no Anta)', 
            'Anta (no Alco)')
rownames(coef) <- cnames
# Name the weights according to groups. 
colnames(coef) <- levels(cells)  
#-------------------------------------------------------------------------------


# Calculate and plot contrast scores with CIs, using get_contrast() ------------
# Below, the error = 'anova2' will pool the error only from groups that are  
# included in the contrast (i.e., from groups that have nonzero weights).
# The error = 'anova' will pool the error from all groups 
# (even if they have zero weights). This is appropriate if all groups have 
# similar variances.
contrast_result <- apply(coef, 1, get_bcontrast, data = nausea, group = cells,
                         error = 'anova', standardset = TRUE)
# If you run "contrast_result" with "standardset = TRUE",
# these contrasts will be converted to standard sets (i.e. absolute weights sum to 2)
# If you run "contrast_result" with "standardset = FALSE",
# these contrasts will be computed with the given weights.
contrast_result <- t(contrast_result)  # Transpose matrix using t(), if you 
                                  # prefer to have each contrast in a single row
print(contrast_result)
plot_contrast(contrast_result)
#-------------------------------------------------------------------------------

# Example results:
#
# contrast_result <- apply(coef, 1, get_bcontrast, data = nausea, group = cells,
#                          error = 'anova', standardset = TRUE)
#
# Note that the dfs and MSerror are identical for all contrasts. The reason is that 
# with "error = anova", the error is pooled from all groups (including those with zero weights).
# Note further that with "standardset = TRUE", the contrast weights are turned into standard sets.
# For example, if you had entered [+3 -1 -1 -1] for the synergistic effect, the weights
# would be changed to [+1 -1/3 -1/3 -1/3], as listed below. In contrast, if you had entered
# [+3 -1 -1 -1] with standardset = FALSE, the contrast score would be 3 times bigger than 
# the 12.21 computed with a standard set. So, keep in mind that the contrast score is scaled
# by the weights!

#                   Score      CI_LL     CI_UL       Tobs  df            p        SE  MSerror
# Alco            6.15000  5.5044210  6.795579 18.8681014 116 3.788579e-37 0.3259469 3.187242
# Anta            6.13000  5.4844210  6.775579 18.8067417 116 5.040661e-37 0.3259469 3.187242
# Alco x Anta     6.03000  5.3844210  6.675579 18.4999433 116 2.116661e-36 0.3259469 3.187242
# Synergistic    12.20667 11.4612163 12.952117 32.4325281 116 5.257663e-60 0.3763711 3.187242
# Alco (no Anta)  0.10000 -0.8129866  1.012987  0.2169392 116 8.286368e-01 0.4609586 3.187242
# Anta (no Alco)  0.12000 -0.7929866  1.032987  0.2603271 116 7.950730e-01 0.4609586 3.187242
#
#                   Anta+Alco noAnta+Alco Anta+noAlco noAnta+noAlco
# Alco                 0.5      -0.500       0.500        -0.500
# Anta                 0.5       0.500      -0.500        -0.500
# Alco x Anta          0.5      -0.500      -0.500         0.500
# Synergistic          1.0      -0.333      -0.333        -0.333
# Alco (no Anta)       0.0       1.000       0.000        -1.000
# Anta (no Alco)       0.0       0.000       1.000        -1.000
