# General information:
# 1) We will assume for the sake of simplicity that the data fulfill all requirements 
# for the use of t tests: Observations are randomly and independently drawn from 
# normally distributed populations of equal variance.
# 2) The 95% CIs refer only to the individual mean (arelational CI) and cannot be used
# directly to draw conclusions about differences between means (i.e., do not use the CIs 
# to compare different contrasts with each other).


# Example of contrast analyses of two-way between-subject data
#
# Data similar to that in Howell (2013, table 13.2) who modelled it after a study by Eysenck (1974).
# Number of recalled words in relation to five levels of encoding, defined according to  
# instructions to participants, and ordered from shallow to deep 
# (C = count letters, R = rhyme, A = think of appropriate adjective, 
#  I = imagine visually, M = memorize for later recall).
#  
#  Here we pretend the design has twoway within-subjects
#  factors: 5 (Encoding method) x 2 (modern / old words), and n = 10 subjects
#
# Stefan Wiens & Mats E. Nilsson, 2016-06-23
# Contact: mats.nilsson@psychology.su.se


# Clear everything -------------------------------------------------------------
rm(list = ls()) # clear memory
graphics.off() # clear all plots
cat("\014")     # clear console (same as Ctrl-L in console)
#-------------------------------------------------------------------------------


# Set working directory (mydir) ------------------------------------------------
mydir <- 'c:/Users/MATNI/Dropbox/Statspaper with Mats/Supplement/R'
setwd(mydir)
# ------------------------------------------------------------------------------


# Load functions ---------------------------------------------------------------
# The file 'functions_contrasts.r' (should be in mydir) contain code for these 
# functions: get_example_data(), get_contrast(), plot_contrast()
source('functions_contrasts.r')  
# ------------------------------------------------------------------------------


# Get example data, taken from Howell, Table 13.2 ------------------------------
d <- get_example_data(design = 'within', way = 'two')
# Define factor with all age x encode groups
d$cells <- interaction(d$encode, d$age)
# ------------------------------------------------------------------------------


# Sreening of data and ---------------------------------------------------------
summary(d)
boxplot(d$recall ~ d$age + d$encode, col = rep(c('white', 'grey'), 5))
aggregate(d$recall, list(d$encode, d$age), summary)
# ------------------------------------------------------------------------------


# Define contrast-weight matrix-------------------------------------------------
# Create matrix with five contrasts  
# If you run "contrast_result" below with "standardset = TRUE",
# these contrasts will be converted to standard sets (i.e. absolute weights sum to 2)
# If you run "contrast_result" below with "standardset = FALSE",
# these contrasts will be computed with the given weights.
coef <-  matrix(c(-2, -1,  0,  1,  2,  # 1. Linear trend (Do numbers of recalled       
                  -2, -1,  0,  1,  2,  # words increase with depth of encoding?)
                  
                  -1, -1, -1, -1, -1,  # 2. Main effect age-group (Are the new 
                   1,  1,  1,  1,  1,  # words recalled more easily than old?)
                  
                   2,  1,  0, -1, -2,  # 3. Trend x Word-age interaction (Is positive 
                  -2, -1,  0,  1,  2,  # trend stronger in new than old words?)
                  
                  -1,  0,  0,  0,  0,  # 4. New versus old words for the 
                   1,  0,  0,  0,  0,  # Count words (C) encoding condition
                  
                   0,  0,  0,  0, -1,  # 5. New versus old words for the 
                   0,  0,  0,  0,  1,  # memorize (M) encoding condition
                
                   1,  0,  0,  0, -1,  # 6. Interaction new vs. old by memorize  
                  -1,  0,  0,  0,  1,  # (M) vs. count (C) condition
                
                   1,  1,  0,  0, -2,  # 7. Interaction new vs. old by memorize  
                  -1, -1,  0,  0,  2), # (M) vs. combined C and R conditions                  
                  nrow = 7, ncol = 10, byrow = TRUE)

# Check if contrast sum to zero (within 1e-12 margin of error)
if (abs(sum(apply(coef,1,sum))) > 1e-12) stop ('Not all contrasts sum to zero!')

# Add row and column names to the contrast-weight matrix 
cnames <- c('Encode trend', 'New-Old','New-Old x Encode trend', 
            'New-Old | C', 'New-Old | M', 'New-Old x M-C', 'New-Old x M-(C+R)')
rownames(coef) <- cnames
colnames(coef) <- levels(d$cells)
#-------------------------------------------------------------------------------


# Calculate and plot contrast scores with CIs, using get_contrast() ------------
# If you run "contrast_result" with "standardset = TRUE",
# these contrasts will be converted to standard sets (i.e. absolute weights sum to 2)
# If you run "contrast_result" with "standardset = FALSE",
# these contrasts will be computed with the given weights.
contrast_result <- apply(coef, 1, get_wcontrast, data = d$recall, 
                         group = d$cells, id = d$id,
                         standardset = FALSE)
contrast_result <- t(contrast_result)  # Transpose matrix using t(), if you 
                                  # prefer to have each contrast in a single row
print(contrast_result)
plot_contrast(contrast_result)
#-------------------------------------------------------------------------------